import { grabPrimaryRequiredDbSchema } from "../../../shell/createDbFromSchema/grab-required-database-schemas";
import { APICreateUserFunctionParams, APIResponseObject } from "../../../types";
import addUsersTableToDb from "../../backend/addUsersTableToDb";
import addDbEntry from "../../backend/db/addDbEntry";
import updateUsersTableSchema from "../../backend/updateUsersTableSchema";
import dbHandler from "../../backend/dbHandler";
import hashPassword from "../../dsql/hashPassword";
import validateEmail from "../../email/fns/validate-email";
import { DSQL_DATASQUIREL_USERS } from "../../../types/dsql";

/**
 * # API Create User
 */
export default async function apiCreateUser({
    encryptionKey,
    payload,
    database,
    dsqlUserID,
    verify,
}: APICreateUserFunctionParams): Promise<APIResponseObject> {
    const finalEncryptionKey =
        encryptionKey || process.env.DSQL_ENCRYPTION_PASSWORD;

    if (!finalEncryptionKey) {
        return {
            success: false,
            msg: "No encryption key provided",
            payload: null,
        };
    }

    if (!finalEncryptionKey?.match(/.{8,}/)) {
        return {
            success: false,
            msg: "Encryption key must be at least 8 characters long",
            payload: null,
        };
    }

    const targetDbSchema = grabPrimaryRequiredDbSchema({
        dbSlug: database,
        userId: dsqlUserID,
        dbId: dsqlUserID ? undefined : 1,
    });

    if (!targetDbSchema?.id) {
        return {
            success: false,
            msg: "targetDbSchema not found",
            payload: null,
        };
    }

    const dbFullName = targetDbSchema.dbFullName;

    if (!dbFullName) {
        return {
            success: false,
            msg: "dbFullName not found",
            payload: null,
        };
    }

    const hashedPassword = hashPassword({
        encryptionKey: finalEncryptionKey,
        password: String(payload.password),
    });

    payload.password = hashedPassword;

    const fieldsQuery = `SHOW COLUMNS FROM ${dbFullName}.users`;

    let fields = (await dbHandler({
        query: fieldsQuery,
        database: dbFullName,
    })) as any[];

    if (!fields?.[0]) {
        const newTable = await addUsersTableToDb({
            userId: dsqlUserID,
            database: dbFullName,
            payload: payload,
            dbId: targetDbSchema.id,
        });

        if (!newTable) {
            return {
                success: false,
                msg: "Could not create users table",
                payload: null,
            };
        }
    }

    const fieldsTitles = fields.map((fieldObject: any) => fieldObject.Field);

    let invalidField = null;

    for (let i = 0; i < Object.keys(payload).length; i++) {
        const key = Object.keys(payload)[i];
        if (!fieldsTitles.includes(key)) {
            await updateUsersTableSchema({
                userId: dsqlUserID,
                database: dbFullName,
                newPayload: {
                    [key]: payload[key],
                },
                dbId: targetDbSchema.id,
            });
        }
    }

    if (invalidField) {
        return {
            success: false,
            msg: `${invalidField} is not a valid field!`,
        };
    }

    const existingUserQuery = `SELECT * FROM ${dbFullName}.users WHERE email = ?${
        payload.username ? " OR username = ?" : ""
    }`;
    const existingUserValues = payload.username
        ? [payload.email, payload.username]
        : [payload.email];

    const existingUser = (await dbHandler({
        query: existingUserQuery,
        values: existingUserValues,
        database: dbFullName,
    })) as any[];

    if (existingUser?.[0]) {
        return {
            success: false,
            msg: "User Already Exists",
            payload: null,
        };
    }

    const isEmailValid = await validateEmail({ email: payload.email });

    if (!isEmailValid.isValid) {
        return {
            success: false,
            msg: isEmailValid.message,
            payload: null,
        };
    }

    const addUser = await addDbEntry<DSQL_DATASQUIREL_USERS>({
        dbFullName: dbFullName,
        tableName: "users",
        data: {
            ...payload,
            image:
                process.env.DSQL_DEFAULT_USER_IMAGE ||
                "/images/user-preset.png",
            image_thumbnail:
                process.env.DSQL_DEFAULT_USER_IMAGE ||
                "/images/user-preset-thumbnail.png",
            verification_status: verify ? 1 : 0,
        },
    });

    if (addUser?.payload?.insertId) {
        const newlyAddedUserQuery = `SELECT id,uuid,first_name,last_name,email,username,image,image_thumbnail,verification_status FROM ${dbFullName}.users WHERE id='${addUser.payload.insertId}'`;

        const newlyAddedUser = (await dbHandler({
            query: newlyAddedUserQuery,
            values: [],
            database: dbFullName,
        })) as any[];

        return {
            ...addUser,
            payload: newlyAddedUser[0],
        };
    } else {
        return {
            ...addUser,
            msg: "Could not create user",
        };
    }
}
